import psycopg2
from psycopg2 import sql
import argparse

schema = "monitoring"
table = "monitor"
# if you change this query you must drop the table!!
query="""select
            datname,pid,application_name,client_addr,client_hostname,
            backend_start, now()-state_change as duration,now() as queried,
            state, query 
        from pg_stat_activity
        where state is not Null"""
DEBUG=False

def setup(conn):
    cur = conn.cursor()
    cur.execute("""SELECT schema_name FROM information_schema.schemata 
    WHERE schema_name = '{}';
    """.format(schema))
    if not cur.rowcount:
        cur.execute(sql.SQL("CREATE SCHEMA {};").format(
            sql.Identifier(schema)))
    if DEBUG: print cur.query
    x = "create table {{}}.{{}} as {};".format(query)
    if DEBUG: print x
    cur.execute(sql.SQL(x).format(
            sql.Identifier(schema),
            sql.Identifier(table)
        ))
    if DEBUG: print cur.query
    cur.close()


def create_connection(user,passwd,dbname, host='127.0.0.1', port=5432):
    if passwd:
        conn = psycopg2.connect(
            "dbname={} user={} password={} port={} host={}".format(
            dbname, user, passwd,port,host))
    else:
#use .pgpass
        conn = psycopg2.connect(
            "dbname={} user={} port={} host={}".format(
            dbname, user, port,host))
    conn.set_session(autocommit=True)
    if conn:
        cur = conn.cursor()
    else:
        raise Exception("No Connection has been created")
    cur.execute("SET application_name = 'Astun Monitor App'")
    cur.execute(sql.SQL("""select * from information_schema.tables where
        table_schema=%s and table_name=%s"""),
            [schema,table])

    if cur.rowcount:
        x = "insert into {{}}.{{}} {};".format(query)
        if DEBUG: print x
        cur.execute(sql.SQL(x).format(
            sql.Identifier(schema),
            sql.Identifier(table)
        ))
        if DEBUG: print cur.query
        cur.close()
    else:
        setup(conn)

    conn.close()



def main():
    parser = argparse.ArgumentParser(description="Monitor who is connected to PostGIS")
    parser.add_argument("-d","--database", help="name of the database to watch",
            required=True)
    parser.add_argument("-u","--user", help="username to use to login with",
            required=True)
    parser.add_argument("-p","--password", help="password to use to login with")
    parser.add_argument("--host", help="hostname of server (default=localhost)",
            default="localhost")
    parser.add_argument("--port", help="port for postgis (default=5432)",
            type=int, default=5432)


    args = parser.parse_args()

    create_connection(user=args.user,passwd=args.password,dbname=args.database,
            host=args.host,port=args.port)

if __name__ == "__main__": main()
